==> openshift-serverless-logic-1.25.0-src.zip: openshift-serverless-logic-1.25.0.GA-src/org.kie.kogito-kogito-runtimes-1.27.0.Final/jbpm/jbpm-flow/src/test/java/org/jbpm/process/core/datatype/impl/coverter/CloneHelperTest.java <==
/*
 * Copyright 2022 Red Hat, Inc. and/or its affiliates.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.jbpm.process.core.datatype.impl.coverter;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Objects;
import java.util.function.UnaryOperator;

import org.junit.jupiter.api.Test;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotSame;
import static org.junit.jupiter.api.Assertions.assertNull;
import static org.junit.jupiter.api.Assertions.assertSame;
import static org.junit.jupiter.api.Assertions.assertThrows;

public class CloneHelperTest {

    private static class CollectionHolder<T> {

        private final Collection<T> collection;

        public Collection<T> getCollection() {
            return collection;
        }

        public CollectionHolder(Collection<T> collection) {
            this.collection = collection;
        }
    }

    private static class CloneableCollectionHolder<T> extends CollectionHolder<T> implements Cloneable {
        public CloneableCollectionHolder(Collection<T> collection) {
            super(collection);
        }

        @Override
        public Object clone() {
            return new CloneableCollectionHolder<>(new ArrayList<>(super.getCollection()));
        }
    }

    private static class DumbCloneableCollectionHolder<T> extends CollectionHolder<T> implements Cloneable {
        public DumbCloneableCollectionHolder(Collection<T> collection) {
            super(collection);
        }

        @Override
        public Object clone() {
            throw new UnsupportedOperationException();
        }
    }

    private static class LierCloneableCollectionHolder<T> extends CollectionHolder<T> implements Cloneable {
        public LierCloneableCollectionHolder(Collection<T> collection) {
            super(collection);
        }
    }

    private static class DumbCopyCollectionHolder<T> extends CollectionHolder<T> {
        public DumbCopyCollectionHolder(Collection<T> collection) {
            super(collection);
        }

        public DumbCopyCollectionHolder(CollectionHolder<T> collection) {
            super(new ArrayList<>(collection.getCollection()));
            throw new RuntimeException();
        }
    }

    private static class CopyCollectionHolder<T> extends CollectionHolder<T> {
        public CopyCollectionHolder(Collection<T> collection) {
            super(collection);
        }

        public CopyCollectionHolder(CopyCollectionHolder<T> collection) {
            super(new ArrayList<>(collection.getCollection()));
        }
    }

    private static class CustomCloneable {
        private final String name;

        public CustomCloneable(String name) {
            this.name = name;
        }

        public String getName() {
            return name;
        }

        @Override
        public int hashCode() {
            return Objects.hash(name);
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj)
                return true;
            if (!(obj instanceof CustomCloneable))
                return false;
            CustomCloneable other = (CustomCloneable) obj;
            return Objects.equals(name, other.name);
        }
    }

    @Test
    void testCloneable() {
        CollectionHolder<Integer> toClone = new CloneableCollectionHolder<>(Arrays.asList(1, 2, 3, 4));
        CollectionHolder<Integer> cloned = (CollectionHolder<Integer>) CloneHelper.clone(toClone);
        assertNotSame(toClone.getCollection(), cloned.getCollection());
        assertEquals(toClone.getCollection(), cloned.getCollection());
    }

    @Test
    void testCopyConstructor() {
        CollectionHolder<Integer> toClone = new CopyCollectionHolder<>(Arrays.asList(1, 2, 3, 4));
        CollectionHolder<Integer> cloned = (CollectionHolder<Integer>) CloneHelper.clone(toClone);
        assertNotSame(toClone.getCollection(), cloned.getCollection());
        assertEquals(toClone.getCollection(), cloned.getCollection());
    }

    @Test
    void testDefault() {
        CollectionHolder<Integer> toClone = new CollectionHolder<>(Arrays.asList(1, 2, 3, 4));
        CollectionHolder<Integer> cloned = (CollectionHolder<Integer>) CloneHelper.clone(toClone);
        assertSame(toClone.getCollection(), cloned.getCollection());
    }

    @Test
    void testCloneableError() {
        CollectionHolder<Integer> toClone = new DumbCloneableCollectionHolder<>(Arrays.asList(1, 2, 3, 4));
        UnaryOperator<Object> cloner = CloneHelper.getCloner(toClone.getClass());
        assertThrows(IllegalStateException.class, () -> cloner.apply(toClone));
    }

    @Test
    void testCopyError() {
        CollectionHolder<Integer> toClone = new DumbCopyCollectionHolder<>(Arrays.asList(1, 2, 3, 4));
        UnaryOperator<Object> cloner = CloneHelper.getCloner(toClone.getClass());
        assertThrows(IllegalStateException.class, () -> cloner.apply(toClone));
    }

    @Test
    void testNoCloneable() {
        CollectionHolder<Integer> toClone = new LierCloneableCollectionHolder<>(Arrays.asList(1, 2, 3, 4));
        assertSame(toClone, CloneHelper.clone(toClone));
    }

    @Test
    void testCloneRegister() {
        CloneHelper.registerCloner(CustomCloneable.class, o -> new CustomCloneable(o.getName() + "_" + o.getName()));
        CustomCloneable toClone = new CustomCloneable("Javierito");
        assertEquals(new CustomCloneable("Javierito_Javierito"), CloneHelper.clone(toClone));
    }

    @Test
    void testCloneIntPrimitive() {
        assertSame(1, CloneHelper.clone(1));
    }

    @Test
    void testCloneBoolPrimitive() {
        assertSame(true, CloneHelper.clone(true));
    }

    @Test
    void testCloneWrapper() {
        Integer integer = Integer.valueOf(1223);
        assertSame(integer, CloneHelper.clone(integer));
    }

    @Test
    void testCloneStringPrimitive() {
        assertEquals("pepe", CloneHelper.clone("pepe"));
    }

    @Test
    void testCloneNull() {
        assertNull(CloneHelper.clone(null));
    }
}

==> openshift-serverless-logic-1.26.0-src.zip: openshift-serverless-logic-1.26.0.GA-src/org.kie.kogito-kogito-runtimes-1.30.0.Final/jbpm/jbpm-flow/src/test/java/org/jbpm/process/core/datatype/impl/coverter/CloneHelperTest.java <==
/*
 * Copyright 2022 Red Hat, Inc. and/or its affiliates.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.jbpm.process.core.datatype.impl.coverter;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Objects;
import java.util.function.UnaryOperator;

import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.kie.kogito.jackson.utils.ObjectMapperFactory;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ObjectNode;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotSame;
import static org.junit.jupiter.api.Assertions.assertNull;
import static org.junit.jupiter.api.Assertions.assertSame;
import static org.junit.jupiter.api.Assertions.assertThrows;

public class CloneHelperTest {

    private static class CollectionHolder<T> {

        private final Collection<T> collection;

        public Collection<T> getCollection() {
            return collection;
        }

        public CollectionHolder(Collection<T> collection) {
            this.collection = collection;
        }

        @Override
        public int hashCode() {
            return Objects.hash(collection);
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj)
                return true;
            if (obj == null)
                return false;
            if (getClass() != obj.getClass())
                return false;
            CollectionHolder other = (CollectionHolder) obj;
            return Objects.equals(collection, other.collection);
        }
    }

    private static class CloneableCollectionHolder<T> extends CollectionHolder<T> implements Cloneable {
        public CloneableCollectionHolder(Collection<T> collection) {
            super(collection);
        }

        @Override
        public Object clone() {
            return new CloneableCollectionHolder<>(new ArrayList<>(super.getCollection()));
        }
    }

    private static class DumbCloneableCollectionHolder<T> extends CollectionHolder<T> implements Cloneable {

        public DumbCloneableCollectionHolder(Collection<T> collection) {
            super(collection);
        }

        @Override
        public Object clone() {
            throw new UnsupportedOperationException();
        }
    }

    private static class LierCloneableCollectionHolder<T> extends CollectionHolder<T> implements Cloneable {
        public LierCloneableCollectionHolder(Collection<T> collection) {
            super(collection);
        }
    }

    private static class DumbCopyCollectionHolder<T> extends CollectionHolder<T> {
        public DumbCopyCollectionHolder(Collection<T> collection) {
            super(collection);
        }

        public DumbCopyCollectionHolder(CollectionHolder<T> collection) {
            super(new ArrayList<>(collection.getCollection()));
            throw new RuntimeException();
        }
    }

    private static class CopyCollectionHolder<T> extends CollectionHolder<T> {
        public CopyCollectionHolder(Collection<T> collection) {
            super(collection);
        }

        public CopyCollectionHolder(CopyCollectionHolder<T> collection) {
            super(new ArrayList<>(collection.getCollection()));
        }
    }

    private static class CustomCloneable {
        private final String name;

        public CustomCloneable(String name) {
            this.name = name;
        }

        public String getName() {
            return name;
        }

        @Override
        public int hashCode() {
            return Objects.hash(name);
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj)
                return true;
            if (!(obj instanceof CustomCloneable))
                return false;
            CustomCloneable other = (CustomCloneable) obj;
            return Objects.equals(name, other.name);
        }
    }

    @BeforeAll
    static void init() {
        CloneHelperRegister.get().registerCloner(CustomCloneable.class, o -> new CustomCloneable(o.getName() + "_" + o.getName()));
    }

    @Test
    void testCloneable() {
        assertCloned(new CloneableCollectionHolder<>(Arrays.asList(1, 2, 3, 4)));
    }

    @Test
    void testCopyConstructor() {
        assertCloned(new CopyCollectionHolder<>(Arrays.asList(1, 2, 3, 4)));
    }

    @Test
    void testDefault() {
        assertNotCloned(new CollectionHolder<>(Arrays.asList(1, 2, 3, 4)));
    }

    @Test
    void testCloneableError() {
        assertCloneError(new DumbCloneableCollectionHolder<>(Arrays.asList(1, 2, 3, 4)));
    }

    @Test
    void testCopyError() {
        assertCloneError(new DumbCopyCollectionHolder<>(Arrays.asList(1, 2, 3, 4)));
    }

    @Test
    void testNoCloneable() {
        assertNotCloned(new LierCloneableCollectionHolder<>(Arrays.asList(1, 2, 3, 4)));
    }

    @Test
    void testCloneRegister() {
        assertEquals(new CustomCloneable("Javierito_Javierito"), CloneHelper.get().clone(new CustomCloneable("Javierito")));
    }

    @Test
    void testCloneIntPrimitive() {
        assertNotCloned(1);
    }

    @Test
    void testCloneBoolPrimitive() {
        assertNotCloned(true);
    }

    @Test
    void testCloneStringPrimitive() {
        assertCloned("pepe");
    }

    @Test
    void testCloneNull() {
        assertNull(CloneHelper.get().clone(null));
    }

    @Test
    void testListClone() {
        assertCloned(new ArrayList<>(Arrays.asList(1, 2, 3)));
    }

    @Test
    void testObjectNodeClone() {
        assertSame(CloneHelper.get().getCloner(JsonNode.class), CloneHelper.get().getCloner(ObjectNode.class));
        assertCloned(ObjectMapperFactory.get().createObjectNode().put("name", "Javierito"));
    }

    private void assertNotCloned(Object toClone) {
        assertSame(toClone, CloneHelper.get().clone(toClone));
    }

    private void assertCloned(Object toClone) {
        Object cloned = CloneHelper.get().clone(toClone);
        assertEquals(toClone, cloned);
        assertNotSame(toClone, cloned);
    }

    private <T> void assertCloneError(T toClone) {
        UnaryOperator<T> cloner = (UnaryOperator<T>) CloneHelper.get().getCloner(toClone.getClass());
        assertThrows(IllegalStateException.class, () -> cloner.apply(toClone));
    }
}
